Class: XMLSecurity::SignedDocument
- Inherits:
-
BaseDocument
- Object
- REXML::Document
- BaseDocument
- XMLSecurity::SignedDocument
- Includes:
- OneLogin::RubySaml::ErrorHandling
- Defined in:
- lib/xml_security.rb
Constant Summary
Constants inherited from BaseDocument
BaseDocument::C14N, BaseDocument::DSIG, BaseDocument::NOKOGIRI_OPTIONS
Instance Attribute Summary collapse
Attributes included from OneLogin::RubySaml::ErrorHandling
Instance Method Summary collapse
-
#initialize(response, errors = []) ⇒ SignedDocument
constructor
A new instance of SignedDocument.
- #validate_document(idp_cert_fingerprint, soft = true, options = {}) ⇒ Object
- #validate_document_with_cert(idp_cert, soft = true, check_malformed_doc = true) ⇒ Object
- #validate_signature(base64_cert, soft = true, check_malformed_doc = true) ⇒ Object
Methods included from OneLogin::RubySaml::ErrorHandling
Methods inherited from BaseDocument
#algorithm, #canon_algorithm, safe_load_xml
Constructor Details
#initialize(response, errors = []) ⇒ SignedDocument
Returns a new instance of SignedDocument.
218 219 220 221 |
# File 'lib/xml_security.rb', line 218 def initialize(response, errors = []) super(response) @errors = errors end |
Instance Attribute Details
#signed_element_id ⇒ Object
223 224 225 |
# File 'lib/xml_security.rb', line 223 def signed_element_id @signed_element_id ||= extract_signed_element_id end |
Instance Method Details
#validate_document(idp_cert_fingerprint, soft = true, options = {}) ⇒ Object
227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 |
# File 'lib/xml_security.rb', line 227 def validate_document(idp_cert_fingerprint, soft = true, = {}) # get cert from response cert_element = REXML::XPath.first( self, "//ds:X509Certificate", { "ds"=>DSIG } ) if cert_element base64_cert = OneLogin::RubySaml::Utils.element_text(cert_element) cert_text = Base64.decode64(base64_cert) begin cert = OpenSSL::X509::Certificate.new(cert_text) rescue OpenSSL::X509::CertificateError => _e return append_error("Document Certificate Error", soft) end if [:fingerprint_alg] fingerprint_alg = XMLSecurity::BaseDocument.new.algorithm([:fingerprint_alg]).new else fingerprint_alg = OpenSSL::Digest::SHA1.new end fingerprint = fingerprint_alg.hexdigest(cert.to_der) # check cert matches registered idp cert if fingerprint != idp_cert_fingerprint.gsub(/[^a-zA-Z0-9]/,"").downcase return append_error("Fingerprint mismatch", soft) end else if [:cert] base64_cert = Base64.encode64([:cert].to_pem) else if soft return false else return append_error("Certificate element missing in response (ds:X509Certificate) and not cert provided at settings", soft) end end end check_malformed_doc = true check_malformed_doc = [:check_malformed_doc] if .key?(:check_malformed_doc) validate_signature(base64_cert, soft, check_malformed_doc) end |
#validate_document_with_cert(idp_cert, soft = true, check_malformed_doc = true) ⇒ Object
271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 |
# File 'lib/xml_security.rb', line 271 def validate_document_with_cert(idp_cert, soft = true, check_malformed_doc = true) # get cert from response cert_element = REXML::XPath.first( self, "//ds:X509Certificate", { "ds"=>DSIG } ) if cert_element base64_cert = OneLogin::RubySaml::Utils.element_text(cert_element) cert_text = Base64.decode64(base64_cert) begin cert = OpenSSL::X509::Certificate.new(cert_text) rescue OpenSSL::X509::CertificateError => _e return append_error("Document Certificate Error", soft) end # check saml response cert matches provided idp cert if idp_cert.to_pem != cert.to_pem return append_error("Certificate of the Signature element does not match provided certificate", soft) end else base64_cert = Base64.encode64(idp_cert.to_pem) end validate_signature(base64_cert, true, check_malformed_doc) end |
#validate_signature(base64_cert, soft = true, check_malformed_doc = true) ⇒ Object
298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 |
# File 'lib/xml_security.rb', line 298 def validate_signature(base64_cert, soft = true, check_malformed_doc = true) begin document = XMLSecurity::BaseDocument.safe_load_xml(self, check_malformed_doc) rescue StandardError => error @errors << error. return false if soft raise ValidationError.new("XML load failed: #{error.message}") end # create a rexml document @working_copy ||= REXML::Document.new(self.to_s).root # get signature node sig_element = REXML::XPath.first( @working_copy, "//ds:Signature", {"ds"=>DSIG} ) # signature method sig_alg_value = REXML::XPath.first( sig_element, "./ds:SignedInfo/ds:SignatureMethod", {"ds"=>DSIG} ) signature_algorithm = algorithm(sig_alg_value) # get signature base64_signature = REXML::XPath.first( sig_element, "./ds:SignatureValue", {"ds" => DSIG} ) signature = Base64.decode64(OneLogin::RubySaml::Utils.element_text(base64_signature)) # canonicalization method canon_algorithm = canon_algorithm REXML::XPath.first( sig_element, './ds:SignedInfo/ds:CanonicalizationMethod', 'ds' => DSIG ) noko_sig_element = document.at_xpath('//ds:Signature', 'ds' => DSIG) noko_signed_info_element = noko_sig_element.at_xpath('./ds:SignedInfo', 'ds' => DSIG) canon_string = noko_signed_info_element.canonicalize(canon_algorithm) noko_sig_element.remove # get signed info signed_info_element = REXML::XPath.first( sig_element, "./ds:SignedInfo", { "ds" => DSIG } ) # get inclusive namespaces inclusive_namespaces = extract_inclusive_namespaces # check digests ref = REXML::XPath.first(signed_info_element, "./ds:Reference", {"ds"=>DSIG}) reference_nodes = document.xpath("//*[@ID=$id]", nil, { 'id' => extract_signed_element_id }) if reference_nodes.length > 1 # ensures no elements with same ID to prevent signature wrapping attack. return append_error("Duplicated IDs found", soft) end hashed_element = reference_nodes[0] canon_algorithm = canon_algorithm REXML::XPath.first( signed_info_element, './ds:CanonicalizationMethod', { "ds" => DSIG } ) canon_algorithm = process_transforms(ref, canon_algorithm) canon_hashed_element = hashed_element.canonicalize(canon_algorithm, inclusive_namespaces) digest_algorithm = algorithm(REXML::XPath.first( ref, "./ds:DigestMethod", { "ds" => DSIG } )) hash = digest_algorithm.digest(canon_hashed_element) encoded_digest_value = REXML::XPath.first( ref, "./ds:DigestValue", { "ds" => DSIG } ) digest_value = Base64.decode64(OneLogin::RubySaml::Utils.element_text(encoded_digest_value)) unless digests_match?(hash, digest_value) return append_error("Digest mismatch", soft) end # get certificate object cert_text = Base64.decode64(base64_cert) cert = OpenSSL::X509::Certificate.new(cert_text) # verify signature unless cert.public_key.verify(signature_algorithm.new, signature, canon_string) return append_error("Key validation error", soft) end return true end |